﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.GroupingVector {
  [Item("Single Grouping Move Maker", "Peforms a single grouping move on a given grouping vector and updates the quality.")]
  [StorableClass]
  public class SingleGroupingMoveMaker : SingleSuccessorOperator, IMoveMaker, IGroupingVectorMoveOperator {
    public override bool CanChangeName {
      get { return false; }
    }
    public ILookupParameter<DoubleValue> QualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["Quality"]; }
    }
    public ILookupParameter<DoubleValue> MoveQualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["MoveQuality"]; }
    }
    public ILookupParameter<SingleGroupingMove> SingleGroupingMoveParameter {
      get { return (ILookupParameter<SingleGroupingMove>)Parameters["SingleGroupingMove"]; }
    }
    public ILookupParameter<GroupingVectorEncoding> GroupingVectorParameter {
      get { return (ILookupParameter<GroupingVectorEncoding>)Parameters["GroupingVector"]; }
    }

    [StorableConstructor]
    protected SingleGroupingMoveMaker(bool deserializing) : base(deserializing) { }
    protected SingleGroupingMoveMaker(SingleGroupingMoveMaker original, Cloner cloner) : base(original, cloner) { }
    public SingleGroupingMoveMaker()
      : base() {
      Parameters.Add(new LookupParameter<DoubleValue>("Quality", "The quality of the solution."));
      Parameters.Add(new LookupParameter<SingleGroupingMove>("SingleGroupingMove", "The move to evaluate."));
      Parameters.Add(new LookupParameter<DoubleValue>("MoveQuality", "The relative quality of the move."));
      Parameters.Add(new LookupParameter<GroupingVectorEncoding>("GroupingVector", "The solution as grouping vector."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SingleGroupingMoveMaker(this, cloner);
    }

    public override IOperation Apply() {
      SingleGroupingMove move = SingleGroupingMoveParameter.ActualValue;
      GroupingVectorEncoding groupingVector = GroupingVectorParameter.ActualValue;
      DoubleValue moveQuality = MoveQualityParameter.ActualValue;
      DoubleValue quality = QualityParameter.ActualValue;

      groupingVector.GroupingVector[move.Index] = move.NewGroup;

      quality.Value = moveQuality.Value;

      return base.Apply();
    }
  }
}
