﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("SwapSequenceMoveAttribute", "Specifies the tabu attributes for a swap sequence move on an multi component vector.")]
  [StorableClass]
  public class SwapSequenceMoveAttribute : MultiComponentVectorMoveAttribute {
    [Storable]
    public int Index1 { get; protected set; }
    [Storable]
    public int Index2 { get; protected set; }
    [Storable]
    public int ItemIndex1 { get; protected set; }
    [Storable]
    public int ItemIndex2 { get; protected set; }

    [StorableConstructor]
    protected SwapSequenceMoveAttribute(bool deserializing) : base(deserializing) { }
    protected SwapSequenceMoveAttribute(SwapSequenceMoveAttribute original, Cloner cloner)
      : base(original, cloner) {
      this.ItemIndex1 = original.ItemIndex1;
      this.ItemIndex2 = original.ItemIndex2;
    }
    public SwapSequenceMoveAttribute() : this(-1, -1, -1, -1, -1) { }
    public SwapSequenceMoveAttribute(int index1, int index2, int itemIndex1, int itemIndex2, double moveQuality)
      : base(moveQuality) {
        Index1 = index1;
        Index2 = index2;
        ItemIndex1 = itemIndex1;
        ItemIndex2 = itemIndex2;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SwapSequenceMoveAttribute(this, cloner);
    }
  }
}
