﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.PackingEncoding.MultiComponentVector;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("SingleGroupingMove", "A move on a multi component vector that is specified by a single index and a new grouping-value. The grouping of the item on the specified index is changed to the new value.")]
  [StorableClass]
  public class SingleGroupingMove : MultiComponentVectorMove {
    [Storable]
    public int ItemIndex { get; protected set; }
    [Storable]
    public int NewGroup { get; protected set; }

    [StorableConstructor]
    protected SingleGroupingMove(bool deserializing) : base(deserializing) { }
    protected SingleGroupingMove(SingleGroupingMove original, Cloner cloner)
      : base(original, cloner) {
      this.ItemIndex = original.ItemIndex;
      this.NewGroup = original.NewGroup;
    }
    public SingleGroupingMove(int index, int newGroup, MultiComponentVectorEncoding multiComponentVector)
      : base(multiComponentVector) {
      ItemIndex = index;
      NewGroup = newGroup;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SingleGroupingMove(this, cloner);
    }

    public override MultiComponentVectorEncoding GetVectorAfterMove() {
      //var result = new MultiComponentVectorEncoding();
      //result.PackingInformations = new ItemList<PackingInformation>(this.MultiComponentVector.PackingInformations);
      //result.PackingInformations[ItemIndex].AssignedBin = NewGroup;
      //return result;
      return InsertItemgeneAfterLastItemgeneOfUsedBin(MultiComponentVector, ItemIndex, NewGroup);
    }
    private MultiComponentVectorEncoding InsertItemgeneAfterLastItemgeneOfUsedBin(MultiComponentVectorEncoding solution, int itemIndex, int binNr) {
      var itemgene = solution.PackingInformations[itemIndex];
      solution.PackingInformations.Remove(itemgene);
      itemgene.AssignedBin = binNr;
      int targetIndex = solution.PackingInformations.FindLastIndex(pi => pi.AssignedBin == binNr);
      solution.PackingInformations.Insert(targetIndex + 1, itemgene);
      return solution;
    }

    public override System.Type GetMoveAttributeType() {
      return typeof(SingleGroupingMoveAttribute);
    }
  }
}