﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.PackingEncoding.MultiComponentVector;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("SingleItemRotationMove", "A move on a multi component vector that is specified by a single index. The item at the specified index is rotated.")]
  [StorableClass]
  public class SingleItemRotationMove : MultiComponentVectorMove {
    [Storable]
    public int ItemIndex { get; protected set; }

    [StorableConstructor]
    protected SingleItemRotationMove(bool deserializing) : base(deserializing) { }
    protected SingleItemRotationMove(SingleItemRotationMove original, Cloner cloner)
      : base(original, cloner) {
        this.ItemIndex = original.ItemIndex;
    }
    public SingleItemRotationMove(int index, MultiComponentVectorEncoding multiComponentVector)
      : base(multiComponentVector) {
        ItemIndex = index;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SingleItemRotationMove(this, cloner);
    }

    public override MultiComponentVectorEncoding GetVectorAfterMove() {
      var result = new MultiComponentVectorEncoding();
      result.PackingInformations = new ItemList<PackingInformation>(this.MultiComponentVector.PackingInformations);
      result.PackingInformations[ItemIndex].Rotated = !result.PackingInformations[ItemIndex].Rotated;
      return result;
    }

    public override System.Type GetMoveAttributeType() {
      return typeof(SingleItemRotationMoveAttribute);
    }
  }
}